/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <folly/portability/GTest.h>
#include <thrift/compiler/codemod/file_manager.h>
#include <thrift/compiler/test/parser_test_helpers.h>
#include <thrift/compiler/util.h>

namespace apache::thrift::compiler {

std::string read_file(const std::string& path) {
  std::string content;
  EXPECT_TRUE(folly::readFile(path.c_str(), content));
  return content;
}

// Testing overloading of < operator in replacement struct.
TEST(FileManagerTest, replacement_less_than) {
  codemod::replacement a{2, 4, ""};
  codemod::replacement b{2, 5, ""};
  codemod::replacement c{3, 5, ""};
  codemod::replacement d{5, 7, ""};

  EXPECT_TRUE(a < b); // Same begin, different end
  EXPECT_TRUE(b < c); // Same end, different begin
  EXPECT_TRUE(a < c); // Overlapping
  EXPECT_TRUE(a < d); // Non-overlapping
}

// Basic test of apply_replacements functionality, without traversing AST.
TEST(FileManagerTest, apply_replacements_test) {
  auto source_mgr = source_manager();
  auto program = dedent_and_parse_to_program(source_mgr, R"(
    struct A {
      1: optional A a (cpp.ref);
    } (cpp.noexcept_move)
  )");

  codemod::file_manager fm(source_mgr, *program);

  fm.add({13, 39, "@cpp.Ref{cpp.RefType.Unique}\n  1: optional string a;"});
  fm.add({41, 61, ""});

  fm.apply_replacements();

  EXPECT_EQ(read_file(program->path()), strip_left_margin(R"(
    struct A {
      @cpp.Ref{cpp.RefType.Unique}
      1: optional string a;
    }
  )"));
}

} // namespace apache::thrift::compiler
